@@ -7,17 +7,18 @@
 /// # Example Usage in a combined BeaconSender and BeaconListener
 /// ```
 /// use simpdiscoverylib::{BeaconSender, BeaconListener};
+/// use std::time::Duration;
 ///
 /// let port = 34254;
 /// let my_service_name = "net.mackenzie-serres.simpdiscovery";
 /// if let Ok(beacon) = BeaconSender::new(port, my_service_name) {
 ///     std::thread::spawn(move || {
-///         let _ = beacon.send_loop();
+///         let _ = beacon.send_loop(Duration::from_secs(1));
 ///     });
 /// }
 ///
-/// let listener = BeaconListener::new(34254).expect("Could not create listener");
-/// let beacon = listener.wait().expect("Failed to receive beacon");
+/// let listener = BeaconListener::new(port, None).expect("Could not create listener");
+/// let beacon = listener.wait(None).expect("Failed to receive beacon");
 /// assert_eq!(beacon.message, my_service_name, "Service name received in beacon doesn't match the one expected");
 /// ```
 
@@ -46,11 +47,12 @@ pub struct BeaconSender {
 ///
 /// ```
 /// use simpdiscoverylib::{BeaconSender, BeaconListener};
+/// use std::time::Duration;
 ///
 /// let port = 34254;
 /// if let Ok(beacon) = BeaconSender::new(port, "Hello") {
 ///     std::thread::spawn(move || {
-///         let _ = beacon.send_loop();
+///         let _ = beacon.send_loop(Duration::from_secs(1));
 ///     });
 /// }
 impl BeaconSender {
@@ -74,10 +76,10 @@ impl BeaconSender {
     }
 
     /// Enter an infinite loop sending `Beacon`s periodically
-    pub fn send_loop(&self) -> std::io::Result<()> {
+    pub fn send_loop(&self, period: Duration) -> std::io::Result<()> {
         loop {
             self.send_one_beacon()?;
-            std::thread::sleep(Duration::from_secs(1));
+            std::thread::sleep(period);
         }
     }
 
@@ -99,35 +101,64 @@ pub struct Beacon {
 /// `BeaconListener` listens for new `Beacons` on the specified port
 pub struct BeaconListener {
     socket: UdpSocket,
+    filter: Option<String>
 }
 
 impl BeaconListener {
     /// Create a new `BeaconListener` on the specified port
-    pub fn new(port: usize) -> std::io::Result<Self> {
+    pub fn new(port: usize, filter: Option<String>) -> std::io::Result<Self> {
         let address = format!("{}:{}", "0.0.0.0", port);
         let socket = UdpSocket::bind(&address)?;
         info!("Socket bound to: {}", address);
 
         Ok(Self {
-            socket
+            socket,
+            filter
         })
     }
 
     /// Wait for a `Beacon` on the port specified in `BeaconListener::new()`
-    pub fn wait(&self) -> std::io::Result<Beacon> {
+    ///
+    /// # Example with timeout
+    ///
+    /// ```
+    /// use simpdiscoverylib::BeaconListener;
+    /// use std::time::Duration;
+    ///
+    /// let port = 34254;
+    /// let listener = BeaconListener::new(port, None).expect("Could not create listener");
+    /// let beacon = listener.wait(Some(Duration::from_millis(1)));
+    /// assert!(beacon.is_err());
+    /// ```
+    pub fn wait(&self, timeout: Option<Duration>) -> std::io::Result<Beacon> {
         let mut buffer = [0; MAX_INCOMING_BEACON_SIZE];
 
-        info!("Waiting for beacon");
-        let (number_of_bytes, source_address) = self.socket.recv_from(&mut buffer)?;
-        let message = String::from_utf8(buffer[..number_of_bytes].to_vec())
-            .map_err(|e| std::io::Error::new(std::io::ErrorKind::Other,
-                     e.to_string())
-            )?;
-        info!("Message '{}' received from Address: '{}'", message, source_address);
+        self.socket.set_read_timeout(timeout)?;
+        info!("Read timeout set to: {:?}", timeout);
 
-        Ok(Beacon{
-            source_ip: source_address.to_string(),
-            message
-        })
+        info!("Waiting for beacon");
+        loop {
+            let (number_of_bytes, source_address) = self.socket.recv_from(&mut buffer)?;
+            let message = String::from_utf8(buffer[..number_of_bytes].to_vec())
+                .map_err(|e| std::io::Error::new(std::io::ErrorKind::Other,
+                                                 e.to_string())
+                )?;
+            info!("Message '{}' received from Address: '{}'", message, source_address);
+
+            match &self.filter {
+                Some(match_string) => {
+                    if &message == match_string {
+                        return Ok(Beacon {
+                            source_ip: source_address.to_string(),
+                            message
+                        });
+                    }
+                },
+                None => return Ok(Beacon {
+                    source_ip: source_address.to_string(),
+                    message
+                })
+            }
+        }
     }
 }
